@@ -34,6 +34,7 @@
 #include "bluetooth-client.h"
 #include "bluetooth-client-private.h"
 #include "bluetooth-client-glue.h"
+#include "gnome-bluetooth-enum-types.h"
 
 #include "marshal.h"
 
@@ -63,6 +64,7 @@
 static char * detectable_interfaces[] = {
 	"org.bluez.Headset",
 	"org.bluez.AudioSink",
+	"org.bluez.Audio",
 	"org.bluez.Input"
 };
 
@@ -100,7 +102,13 @@ enum {
 
 G_DEFINE_TYPE(BluetoothClient, bluetooth_client, G_TYPE_OBJECT)
 
-const gchar *bluetooth_type_to_string(guint type)
+/**
+ * bluetooth_type_to_string:
+ * @type: a #BluetoothType
+ *
+ * Return value: the string representation of the @type passed. Do not free the return value.
+ **/
+const gchar *bluetooth_type_to_string(BluetoothType type)
 {
 	switch (type) {
 	case BLUETOOTH_TYPE_ANY:
@@ -137,6 +145,12 @@ const gchar *bluetooth_type_to_string(guint type)
 	}
 }
 
+/**
+ * bluetooth_verify_address:
+ * @bdaddr: a string representing a Bluetooth address
+ *
+ * Return value: Whether the string is a valid Bluetooth address. This does not contact the device in any way.
+ **/
 gboolean
 bluetooth_verify_address (const char *bdaddr)
 {
@@ -169,7 +183,8 @@ bluetooth_verify_address (const char *bdaddr)
 	return retval;
 }
 
-static guint class_to_type(guint32 class)
+guint
+bluetooth_class_to_type (guint32 class)
 {
 	switch ((class & 0x1f00) >> 8) {
 	case 0x01:
@@ -309,12 +324,24 @@ device_services_changed (DBusGProxy *iface, const char *property,
 	GtkTreePath *tree_path;
 	GHashTable *table;
 	const char *path;
-	gboolean is_connected;
+	BluetoothStatus status;
 
 	if (g_str_equal (property, "Connected") != FALSE) {
-		is_connected = g_value_get_boolean(value);
+		status = g_value_get_boolean (value) ?
+			BLUETOOTH_STATUS_CONNECTED :
+			BLUETOOTH_STATUS_DISCONNECTED;
 	} else if (g_str_equal (property, "State") != FALSE) {
-		is_connected = (g_strcmp0(g_value_get_string (value), "connected") == 0);
+		GEnumClass *eclass;
+		GEnumValue *ev;
+		eclass = g_type_class_ref (BLUETOOTH_TYPE_STATUS);
+		ev = g_enum_get_value_by_nick (eclass, g_value_get_string (value));
+		if (ev == NULL) {
+			g_warning ("Unknown status '%s'", g_value_get_string (value));
+			status = BLUETOOTH_STATUS_DISCONNECTED;
+		} else {
+			status = ev->value;
+		}
+		g_type_class_unref (eclass);
 	} else
 		return;
 
@@ -328,7 +355,7 @@ device_services_changed (DBusGProxy *iface, const char *property,
 
 	g_hash_table_insert (table,
 			     (gpointer) dbus_g_proxy_get_interface (iface),
-			     GINT_TO_POINTER (is_connected));
+			     GINT_TO_POINTER (status));
 
 	tree_path = gtk_tree_model_get_path (GTK_TREE_MODEL (priv->store), &iter);
 	gtk_tree_model_row_changed (GTK_TREE_MODEL (priv->store), tree_path, &iter);
@@ -349,22 +376,24 @@ device_list_nodes (DBusGProxy *device, BluetoothClient *client, gboolean connect
 	for (i = 0; i < G_N_ELEMENTS (detectable_interfaces); i++) {
 		DBusGProxy *iface;
 		GHashTable *props;
-		const char *iface_name;
 
 		/* Don't add the input interface for devices that already have
 		 * audio stuff */
 		if (g_str_equal (detectable_interfaces[i], BLUEZ_INPUT_INTERFACE)
 		    && g_hash_table_size (table) > 0)
 			continue;
-		/* Add org.bluez.Audio if the device supports headset or audiosink */
-		if (g_str_equal (detectable_interfaces[i], BLUEZ_HEADSET_INTERFACE) ||
-		    g_str_equal (detectable_interfaces[i], BLUEZ_AUDIOSINK_INTERFACE))
-		    	iface_name = BLUEZ_AUDIO_INTERFACE;
-		else
-			iface_name = detectable_interfaces[i];
+
+		/* Don't add the audio interface if there's no Headset or AudioSink,
+		 * that means that it could only receive audio */
+		if (g_str_equal (detectable_interfaces[i], BLUEZ_AUDIO_INTERFACE)) {
+			if (g_hash_table_lookup (table, BLUEZ_HEADSET_INTERFACE) == NULL &&
+			    g_hash_table_lookup (table, BLUEZ_AUDIOSINK_INTERFACE) == NULL) {
+				continue;
+			}
+		}
 
 		/* And skip interface if it's already in the hash table */
-		if (g_hash_table_lookup (table, iface_name) != NULL)
+		if (g_hash_table_lookup (table, detectable_interfaces[i]) != NULL)
 			continue;
 
 		iface = dbus_g_proxy_new_from_proxy (device, detectable_interfaces[i], NULL);
@@ -372,21 +401,30 @@ device_list_nodes (DBusGProxy *device, BluetoothClient *client, gboolean connect
 				       G_TYPE_INVALID, dbus_g_type_get_map ("GHashTable", G_TYPE_STRING, G_TYPE_VALUE), &props,
 				       G_TYPE_INVALID) != FALSE) {
 			GValue *value;
-			gboolean is_connected;
+			BluetoothStatus status;
 
 			value = g_hash_table_lookup(props, "Connected");
 			if (value != NULL) {
-				is_connected = g_value_get_boolean(value);
+				status = g_value_get_boolean(value) ?
+					BLUETOOTH_STATUS_CONNECTED :
+					BLUETOOTH_STATUS_DISCONNECTED;
 			} else {
-				const char *str = "disconnected";
-				value = g_hash_table_lookup(props, "State");
-				if (value != NULL)
-					str = g_value_get_string(value);
+				GEnumClass *eclass;
+				GEnumValue *ev;
 
-				is_connected = (g_strcmp0(str, "connected") == 0);
+				eclass = g_type_class_ref (BLUETOOTH_TYPE_STATUS);
+				value = g_hash_table_lookup(props, "State");
+				ev = g_enum_get_value_by_nick (eclass, g_value_get_string (value));
+				if (ev == NULL) {
+					g_warning ("Unknown status '%s'", g_value_get_string (value));
+					status = BLUETOOTH_STATUS_DISCONNECTED;
+				} else {
+					status = ev->value;
+				}
+				g_type_class_unref (eclass);
 			}
 
-			g_hash_table_insert (table, (gpointer) iface_name, GINT_TO_POINTER (is_connected));
+			g_hash_table_insert (table, (gpointer) detectable_interfaces[i], GINT_TO_POINTER (status));
 
 			if (connect_signal != FALSE) {
 				dbus_g_proxy_add_signal(iface, "PropertyChanged",
@@ -482,6 +520,12 @@ uuid16_to_string (guint uuid16, const char *uuid)
 	}
 }
 
+/**
+ * bluetooth_uuid_to_string:
+ * @uuid: a string representing a Bluetooth UUID
+ *
+ * Return value: a string representing a human-readable (but not usable for display to users) version of the @uuid. Do not free the return value.
+ **/
 const char *
 bluetooth_uuid_to_string (const char *uuid)
 {
@@ -638,7 +682,7 @@ static void add_device(DBusGProxy *adapter, GtkTreeIter *parent,
 		name = value ? g_value_get_string(value) : NULL;
 
 		value = g_hash_table_lookup(hash, "Class");
-		type = class_to_type(g_value_get_uint(value));
+		type = value ? bluetooth_class_to_type(g_value_get_uint(value)) : BLUETOOTH_TYPE_ANY;
 
 		value = g_hash_table_lookup(hash, "Icon");
 		icon = value ? g_value_get_string(value) : "bluetooth";
@@ -683,9 +727,13 @@ static void add_device(DBusGProxy *adapter, GtkTreeIter *parent,
 					BLUETOOTH_COLUMN_TYPE, type,
 					BLUETOOTH_COLUMN_ICON, icon,
 					BLUETOOTH_COLUMN_RSSI, rssi,
-					BLUETOOTH_COLUMN_UUIDS, uuids,
 					BLUETOOTH_COLUMN_LEGACYPAIRING, legacypairing,
 					-1);
+			if (uuids != NULL) {
+				gtk_tree_store_set(priv->store, &iter,
+						   BLUETOOTH_COLUMN_UUIDS, uuids,
+						   -1);
+			}
 
 			if (device != NULL) {
 				services = device_list_nodes (device, client, FALSE);
@@ -800,6 +848,18 @@ static void adapter_changed(DBusGProxy *adapter, const char *property,
 		gtk_tree_store_set(priv->store, &iter,
 				BLUETOOTH_COLUMN_DISCOVERING, discovering, -1);
 		notify = TRUE;
+	} else if (g_str_equal(property, "Powered") == TRUE) {
+		gboolean powered = g_value_get_boolean(value);
+		gboolean is_default;
+
+		gtk_tree_store_set(priv->store, &iter,
+				   BLUETOOTH_COLUMN_POWERED, powered, -1);
+		gtk_tree_model_get(GTK_TREE_MODEL(priv->store), &iter,
+				   BLUETOOTH_COLUMN_DEFAULT, &is_default, -1);
+		if (is_default != FALSE && powered != priv->default_adapter_powered) {
+			priv->default_adapter_powered = powered;
+			g_object_notify (G_OBJECT (client), "default-adapter-powered");
+		}
 	}
 
 	if (notify != FALSE) {
@@ -945,7 +1005,9 @@ static void adapter_removed(DBusGProxy *manager,
 	if (gtk_tree_model_iter_n_children (GTK_TREE_MODEL(priv->store), NULL) == 0) {
 		g_free(priv->default_adapter);
 		priv->default_adapter = NULL;
+		priv->default_adapter_powered = FALSE;
 		g_object_notify (G_OBJECT (client), "default-adapter");
+		g_object_notify (G_OBJECT (client), "default-adapter-powered");
 	}
 }
 
@@ -1154,6 +1216,11 @@ static void bluetooth_client_class_init(BluetoothClientClass *klass)
 	}
 }
 
+/**
+ * bluetooth_client_new:
+ *
+ * Return value: a reference to the #BluetoothClient singleton. Unref the object when done.
+ **/
 BluetoothClient *bluetooth_client_new(void)
 {
 	static BluetoothClient *bluetooth_client = NULL;
@@ -1170,6 +1237,12 @@ BluetoothClient *bluetooth_client_new(void)
 	return bluetooth_client;
 }
 
+/**
+ * bluetooth_client_get_model:
+ * @client: a #BluetoothClient object
+ *
+ * Return value: an unfiltered #GtkTreeModel representing the adapter and devices available on the system.
+ **/
 GtkTreeModel *bluetooth_client_get_model (BluetoothClient *client)
 {
 	BluetoothClientPrivate *priv;
@@ -1185,6 +1258,15 @@ GtkTreeModel *bluetooth_client_get_model (BluetoothClient *client)
 	return model;
 }
 
+/**
+ * bluetooth_client_get_filter_model:
+ * @client: a #BluetoothClient object
+ * @func: a #GtkTreeModelFilterVisibleFunc
+ * @data: user data to pass to gtk_tree_model_filter_set_visible_func()
+ * @destroy: a destroy function for gtk_tree_model_filter_set_visible_func()
+ *
+ * Return value: a #GtkTreeModel of devices filtered using the @func, @data and @destroy arguments to pass to gtk_tree_model_filter_set_visible_func().
+ **/
 GtkTreeModel *bluetooth_client_get_filter_model (BluetoothClient *client,
 						 GtkTreeModelFilterVisibleFunc func,
 						 gpointer data, GDestroyNotify destroy)
@@ -1224,6 +1306,12 @@ static gboolean adapter_filter(GtkTreeModel *model,
 	return active;
 }
 
+/**
+ * bluetooth_client_get_adapter_model:
+ * @client: a #BluetoothClient object
+ *
+ * Return value: a filtered #GtkTreeModel with only adapters present.
+ **/
 GtkTreeModel *bluetooth_client_get_adapter_model (BluetoothClient *client)
 {
 	DBG("client %p", client);
@@ -1232,6 +1320,13 @@ GtkTreeModel *bluetooth_client_get_adapter_model (BluetoothClient *client)
 						  NULL, NULL);
 }
 
+/**
+ * bluetooth_client_get_device_model:
+ * @client: a #BluetoothClient object
+ * @adapter: a #DBusGProxy of the adapter object, or %NULL to get the default adapter.
+ *
+ * Return value: a filtered #GtkTreeModel with only devices belonging to the selected adapter listed. Note that the model will follow a specific adapter, and will not follow the default-adapter when %NULL is passed.
+ **/
 GtkTreeModel *bluetooth_client_get_device_model (BluetoothClient *client,
 						 DBusGProxy *adapter)
 {
@@ -1284,6 +1379,15 @@ GtkTreeModel *bluetooth_client_get_device_model (BluetoothClient *client,
 	return model;
 }
 
+/**
+ * bluetooth_client_get_device_filter_model:
+ * @client: a #BluetoothClient object
+ * @func: a #GtkTreeModelFilterVisibleFunc
+ * @data: user data to pass to gtk_tree_model_filter_set_visible_func()
+ * @destroy: a destroy function for gtk_tree_model_filter_set_visible_func()
+ *
+ * Return value: a #GtkTreeModel of adapters filtered using the @func, @data and @destroy arguments to pass to gtk_tree_model_filter_set_visible_func().
+ **/
 GtkTreeModel *bluetooth_client_get_device_filter_model(BluetoothClient *client,
 		DBusGProxy *adapter, GtkTreeModelFilterVisibleFunc func,
 				gpointer data, GDestroyNotify destroy)
@@ -1302,6 +1406,12 @@ GtkTreeModel *bluetooth_client_get_device_filter_model(BluetoothClient *client,
 	return model;
 }
 
+/**
+ * bluetooth_client_get_default_adapter:
+ * @client: a #BluetoothClient object
+ *
+ * Return value: a #DBusGProxy object representing the default adapter, or %NULL if no adapters are present.
+ **/
 DBusGProxy *bluetooth_client_get_default_adapter(BluetoothClient *client)
 {
 	BluetoothClientPrivate *priv = BLUETOOTH_CLIENT_GET_PRIVATE(client);
@@ -1335,6 +1445,14 @@ DBusGProxy *bluetooth_client_get_default_adapter(BluetoothClient *client)
 	return NULL;
 }
 
+/**
+ * bluetooth_client_start_discovery:
+ * @client: a #BluetoothClient object
+ *
+ * Start a discovery on the default adapter.
+ *
+ * Return value: Whether a discovery was successfully started on the default adapter.
+ **/
 gboolean bluetooth_client_start_discovery(BluetoothClient *client)
 {
 	DBusGProxy *adapter;
@@ -1354,6 +1472,14 @@ gboolean bluetooth_client_start_discovery(BluetoothClient *client)
 	return TRUE;
 }
 
+/**
+ * bluetooth_client_stop_discovery:
+ * @client: a #BluetoothClient object
+ *
+ * Stop a discovery started on the default adapter.
+ *
+ * Return value: Whether a discovery was successfully stopped on the default adapter.
+ **/
 gboolean bluetooth_client_stop_discovery(BluetoothClient *client)
 {
 	DBusGProxy *adapter;
@@ -1408,7 +1534,6 @@ gboolean bluetooth_client_create_device(BluetoothClient *client,
 {
 	CreateDeviceData *devdata;
 	DBusGProxy *adapter;
-	DBusGProxyCall *call;
 
 	g_return_val_if_fail (BLUETOOTH_IS_CLIENT (client), FALSE);
 	g_return_val_if_fail (address != NULL, FALSE);
@@ -1428,17 +1553,17 @@ gboolean bluetooth_client_create_device(BluetoothClient *client,
 	devdata->client = g_object_ref (client);
 
 	if (agent != NULL)
-		call = dbus_g_proxy_begin_call_with_timeout(adapter,
-				"CreatePairedDevice", create_device_callback,
-				devdata, g_free, 90 * 1000,
-				G_TYPE_STRING, address,
-				DBUS_TYPE_G_OBJECT_PATH, agent,
-				G_TYPE_STRING, "DisplayYesNo", G_TYPE_INVALID);
+		dbus_g_proxy_begin_call_with_timeout(adapter,
+						     "CreatePairedDevice", create_device_callback,
+						     devdata, g_free, 90 * 1000,
+						     G_TYPE_STRING, address,
+						     DBUS_TYPE_G_OBJECT_PATH, agent,
+						     G_TYPE_STRING, "DisplayYesNo", G_TYPE_INVALID);
 	else
-		call = dbus_g_proxy_begin_call_with_timeout(adapter,
-				"CreateDevice", create_device_callback,
-				devdata, g_free, 60 * 1000,
-				G_TYPE_STRING, address, G_TYPE_INVALID);
+		dbus_g_proxy_begin_call_with_timeout(adapter,
+						     "CreateDevice", create_device_callback,
+						     devdata, g_free, 60 * 1000,
+						     G_TYPE_STRING, address, G_TYPE_INVALID);
 
 	return TRUE;
 }
@@ -1610,6 +1735,10 @@ service_to_index (const char *service)
 		if (g_str_equal (connectable_interfaces[i], service) != FALSE)
 			return i;
 	}
+	for (i = 0; i < G_N_ELEMENTS (detectable_interfaces); i++) {
+		if (g_str_equal (detectable_interfaces[i], service) != FALSE)
+			return i + G_N_ELEMENTS (connectable_interfaces);
+	}
 
 	g_assert_not_reached ();
 
@@ -1690,3 +1819,98 @@ gboolean bluetooth_client_disconnect_service (BluetoothClient *client,
 	return TRUE;
 }
 
+#define BOOL_STR(x) (x ? "True" : "False")
+
+static void
+services_foreach (const char *service, gpointer _value, GString *str)
+{
+	gboolean value = GPOINTER_TO_INT (_value);
+	g_string_append_printf (str, "%s (%s) ", service, value ? "connected" : "not connected");
+}
+
+void
+bluetooth_client_dump_device (GtkTreeModel *model,
+			      GtkTreeIter *iter,
+			      gboolean recurse)
+{
+	DBusGProxy *proxy;
+	char *address, *alias, *name, *icon, **uuids;
+	gboolean is_default, paired, trusted, connected, discovering, powered, is_adapter;
+	GHashTable *services;
+	GtkTreeIter parent;
+	guint type;
+
+	gtk_tree_model_get (model, iter,
+			    BLUETOOTH_COLUMN_ADDRESS, &address,
+			    BLUETOOTH_COLUMN_ALIAS, &alias,
+			    BLUETOOTH_COLUMN_NAME, &name,
+			    BLUETOOTH_COLUMN_TYPE, &type,
+			    BLUETOOTH_COLUMN_ICON, &icon,
+			    BLUETOOTH_COLUMN_DEFAULT, &is_default,
+			    BLUETOOTH_COLUMN_PAIRED, &paired,
+			    BLUETOOTH_COLUMN_TRUSTED, &trusted,
+			    BLUETOOTH_COLUMN_CONNECTED, &connected,
+			    BLUETOOTH_COLUMN_DISCOVERING, &discovering,
+			    BLUETOOTH_COLUMN_POWERED, &powered,
+			    BLUETOOTH_COLUMN_SERVICES, &services,
+			    BLUETOOTH_COLUMN_UUIDS, &uuids,
+			    BLUETOOTH_COLUMN_PROXY, &proxy,
+			    -1);
+	is_adapter = !gtk_tree_model_iter_parent (model, &parent, iter);
+
+	if (is_adapter != FALSE) {
+		/* Adapter */
+		g_print ("Adapter: %s (%s)\n", name, address);
+		if (is_default)
+			g_print ("\tDefault adapter\n");
+		if (discovering)
+			g_print ("\tDiscovery in progress\n");
+		g_print ("\t%s\n", powered ? "Is powered" : "Is not powered");
+	} else {
+		/* Device */
+		g_print ("Device: %s (%s)\n", alias, address);
+		g_print ("\tD-Bus Path: %s\n", proxy ? dbus_g_proxy_get_path (proxy) : "(none)");
+		g_print ("\tType: %s Icon: %s\n", bluetooth_type_to_string (type), icon);
+		g_print ("\tPaired: %s Trusted: %s Connected: %s\n", BOOL_STR(paired), BOOL_STR(trusted), BOOL_STR(connected));
+		if (services != NULL) {
+			GString *str;
+
+			str = g_string_new (NULL);
+			g_hash_table_foreach (services, (GHFunc) services_foreach, str);
+			g_print ("\tServices: %s\n", str->str);
+			g_string_free (str, TRUE);
+		}
+		if (uuids != NULL) {
+			guint i;
+			g_print ("\tUUIDs: ");
+			for (i = 0; uuids[i] != NULL; i++)
+				g_print ("%s ", uuids[i]);
+			g_print ("\n");
+		}
+	}
+	g_print ("\n");
+
+	g_free (alias);
+	g_free (address);
+	g_free (icon);
+	if (proxy != NULL)
+		g_object_unref (proxy);
+	if (services != NULL)
+		g_hash_table_unref (services);
+	g_strfreev (uuids);
+
+	if (recurse == FALSE)
+		return;
+
+	if (is_adapter != FALSE) {
+		GtkTreeIter child;
+
+		if (gtk_tree_model_iter_children (model, &child, iter) == FALSE)
+			return;
+		bluetooth_client_dump_device (model, &child, FALSE);
+		while (gtk_tree_model_iter_next (model, &child))
+			bluetooth_client_dump_device (model, &child, FALSE);
+	}
+
+}
+
