@@ -35,6 +35,7 @@
 #include <bluetooth-client-private.h>
 #include <bluetooth-killswitch.h>
 #include <bluetooth-chooser.h>
+#include <bluetooth-chooser-private.h>
 #include <bluetooth-plugin-manager.h>
 
 #include "adapter.h"
@@ -169,75 +170,22 @@ static void wizard_callback(GtkWidget *button, gpointer user_data)
 		g_printerr("Couldn't execute command: %s\n", command);
 }
 
-static gboolean show_confirm_dialog(const char *name)
-{
-	GtkWidget *dialog;
-	gint response;
-
-	dialog = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL,
-					 GTK_MESSAGE_QUESTION, GTK_BUTTONS_NONE,
-					 _("Remove '%s' from the list of devices?"), name);
-	g_object_set (G_OBJECT (dialog), "secondary-text",
-		      _("If you remove the device, you will have to set it up again before next use."),
-		      NULL);
-
-	gtk_dialog_add_button (GTK_DIALOG (dialog), GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL);
-	gtk_dialog_add_button (GTK_DIALOG (dialog), GTK_STOCK_REMOVE, GTK_RESPONSE_ACCEPT);
-
-	response = gtk_dialog_run (GTK_DIALOG (dialog));
-
-	gtk_widget_destroy (dialog);
-
-	if (response == GTK_RESPONSE_ACCEPT)
-		return TRUE;
-
-	return FALSE;
-}
-
 static void remove_callback(GtkWidget *button, gpointer user_data)
 {
 	adapter_data *adapter = user_data;
-	GValue value = { 0, };
-	DBusGProxy *device;
-	const char *path;
-	char *address, *name;
+	char *address;
 
-	if (bluetooth_chooser_get_selected_device_info (BLUETOOTH_CHOOSER (adapter->chooser),
-							"proxy", &value) == FALSE) {
-		return;
-	}
 	address = bluetooth_chooser_get_selected_device (BLUETOOTH_CHOOSER (adapter->chooser));
-	name = bluetooth_chooser_get_selected_device_name (BLUETOOTH_CHOOSER (adapter->chooser));
-	if (address == NULL || name == NULL) {
-		g_value_unset (&value);
+	if (address == NULL) {
 		g_free (address);
-		g_free (name);
 		return;
 	}
-	device = g_value_dup_object (&value);
-	g_value_unset (&value);
 
-	if (device == NULL)
-		return;
-
-	path = dbus_g_proxy_get_path(device);
-
-	if (show_confirm_dialog (name) == TRUE) {
-		GError *err = NULL;
-		if (dbus_g_proxy_call (adapter->proxy, "RemoveDevice", &err,
-				       DBUS_TYPE_G_OBJECT_PATH, path,
-				       G_TYPE_INVALID, G_TYPE_INVALID) == FALSE) {
-			g_warning ("Failed to remove device %s: %s", address,
-				   err->message);
-			g_error_free (err);
-		} else {
-			bluetooth_plugin_manager_device_deleted (address);
-		}
-	}
+	if (bluetooth_chooser_remove_selected_device (BLUETOOTH_CHOOSER (adapter->chooser)))
+		bluetooth_plugin_manager_device_deleted (address);
+		
 
 	g_free (address);
-	g_free (name);
-	g_object_unref(device);
 }
 
 static void disconnect_callback(GtkWidget *button, gpointer user_data)
@@ -283,7 +231,7 @@ static void create_adapter(adapter_data *adapter)
 	GHashTable *hash = NULL;
 	GValue *value;
 	DBusGProxy *default_proxy;
-	const gchar *address, *name;
+	const gchar *name;
 	gboolean powered, discoverable;
 	guint timeout;
 
@@ -304,9 +252,6 @@ static void create_adapter(adapter_data *adapter)
 				&hash, G_TYPE_INVALID);
 
 	if (hash != NULL) {
-		value = g_hash_table_lookup(hash, "Address");
-		address = value ? g_value_get_string(value) : NULL;
-
 		value = g_hash_table_lookup(hash, "Name");
 		name = value ? g_value_get_string(value) : NULL;
 
@@ -319,7 +264,6 @@ static void create_adapter(adapter_data *adapter)
 		value = g_hash_table_lookup(hash, "DiscoverableTimeout");
 		timeout = value ? g_value_get_uint(value) : 0;
 	} else {
-		address = NULL;
 		name = NULL;
 		powered = FALSE;
 		discoverable = FALSE;
@@ -349,7 +293,7 @@ static void create_adapter(adapter_data *adapter)
 	gtk_box_pack_start(GTK_BOX(mainbox), vbox, FALSE, TRUE, 0);
 
 	/* The discoverable checkbox */
-	button = gtk_check_button_new_with_mnemonic (_("Make computer _discoverable"));
+	button = gtk_check_button_new_with_mnemonic (_("Make computer _visible"));
 	if (powered == FALSE)
 		discoverable = FALSE;
 	if (discoverable != FALSE && timeout == 0)
@@ -434,7 +378,7 @@ static void create_adapter(adapter_data *adapter)
 	gtk_table_attach(GTK_TABLE(table), buttonbox, 1, 2, 1, 2,
 			 GTK_FILL, GTK_FILL, 6, 6);
 
-	button = gtk_button_new_with_label(_("Setup _new device..."));
+	button = gtk_button_new_with_mnemonic(_("Set up _new device..."));
 	image = gtk_image_new_from_stock(GTK_STOCK_ADD,
 						GTK_ICON_SIZE_BUTTON);
 	gtk_button_set_image(GTK_BUTTON(button), image);
